<?php
/**
* @version 			SEBLOD 3.x More
* @package			SEBLOD (App Builder & CCK) // SEBLOD nano (Form Builder)
* @url				https://www.seblod.com
* @editor			Octopoos - www.octopoos.com
* @copyright		Copyright (C) 2009 - 2018 SEBLOD. All Rights Reserved.
* @license 			GNU General Public License version 2 or later; see _LICENSE.php
**/

defined( '_JEXEC' ) or die;

// Plugin
class plgCCK_FieldAddress_To_Coordinates_Osm extends JCckPluginField
{
	protected static $type		=	'address_to_coordinates_osm';
	protected static $path;
	
	// -------- -------- -------- -------- -------- -------- -------- -------- // Construct
	
	// onCCK_FieldConstruct
	public function onCCK_FieldConstruct( $type, &$data = array() )
	{
		if ( self::$type != $type ) {
			return;
		}
		parent::g_onCCK_FieldConstruct( $data );
	}
	
	// -------- -------- -------- -------- -------- -------- -------- -------- // Prepare
	
	// onCCK_FieldPrepareContent
	public function onCCK_FieldPrepareContent( &$field, $value = '', &$config = array() )
	{
		if ( self::$type != $field->type ) {
			return;
		}
		parent::g_onCCK_FieldPrepareContent( $field, $config );
		
		// Set
		$field->value	=	$value;
	}
	
	// onCCK_FieldPrepareForm
	public function onCCK_FieldPrepareForm( &$field, $value = '', &$config = array(), $inherit = array(), $return = false )
	{
		if ( self::$type != $field->type ) {
			return;
		}
		self::$path	=	parent::g_getPath( self::$type.'/' );
		parent::g_onCCK_FieldPrepareForm( $field, $config );

		// Init
		if ( count( $inherit ) ) {
			$id		=	( isset( $inherit['id'] ) && $inherit['id'] != '' ) ? $inherit['id'] : $field->name;
			$name	=	( isset( $inherit['name'] ) && $inherit['name'] != '' ) ? $inherit['name'] : $field->name;
		} else {
			$id		=	$field->name;
			$name	=	$field->name;
		}
		$value		=	( $value != '' ) ? $value : $field->defaultvalue;
		$value		=	( $value != ' ' ) ? $value : '';
		$value		=	htmlspecialchars( $value );
		
		// Validate
		$validate	=	'';
		if ( $config['doValidation'] > 1 ) {
			plgCCK_Field_ValidationRequired::onCCK_Field_ValidationPrepareForm( $field, $id, $config );
			parent::g_onCCK_FieldPrepareForm_Validation( $field, $id, $config );
			$validate	=	( count( $field->validate ) ) ? ' validate['.implode( ',', $field->validate ).']' : '';
		}
		
		// Prepare
		if ( $field->bool == 0 ) {
			$form	=	'';
		} else {
			$class	=	'inputbox text'.$validate . ( $field->css ? ' '.$field->css : '' );
			if ( $value != '' ) {
				$class	.=	' has-value';
			}
			$maxlen	=	( $field->maxlength > 0 ) ? ' maxlength="'.$field->maxlength.'"' : '';
			$attr	=	'class="'.$class.'" size="'.$field->size.'"'.$maxlen . ( $field->attributes ? ' '.$field->attributes : '' );
			$form	=	'<input type="text" id="'.$id.'" name="'.$name.'" value="'.$value.'" '.$attr.' />';
		}

		// Set
		if ( !parent::g_isStaticVariation( $field, $field->variation ) ) {
			$field->form	=	$form;
			if ( $field->bool == 1 ) {
				$options2	=	new JRegistry( $field->options2 );

				if ( $field->state ) {
					parent::g_addProcess(
						'beforeRenderForm',
						self::$type, $config, 
						array(
							'name'=>$field->name,
							'id'=>$id,
							'variation'=>$field->variation,
							// 'api_key'=>trim($this->params->get( 'api_key', '' )),
							'bypass'=>'0'/* $options2->get( 'bypass', '0' ) */,
							'lat'=>$options2->get( 'latitude' ),
							'lng'=>$options2->get( 'longitude' ),
							'postal_code'=>$options2->get( 'postal_code' ),
							'city'=>$options2->get( 'city' ),
							'country'=>$options2->get( 'country' ),
							'country_type'=>$options2->get( 'country_type', '0' ),
							'r_type'=>$options2->get( 'types' ),
							'r_country'=>$options2->get( 'restrictions_country' )
						) 
					);
				}
			}
			if ( $field->script ) {
				parent::g_addScriptDeclaration( $field->script );
			}
		} else {
			parent::g_getDisplayVariation( $field, $field->variation, $value, $value, $form, $id, $name, '<input', '', '', $config );
		}
		$field->value	=	$value;
		
		// Return
		if ( $return === true ) {
			return $field;
		}
	}
	
	// onCCK_FieldPrepareSearch
	public function onCCK_FieldPrepareSearch( &$field, $value = '', &$config = array(), $inherit = array(), $return = false )
	{
		if ( self::$type != $field->type ) {
			return;
		}
		
		// Prepare
		self::onCCK_FieldPrepareForm( $field, $value, $config, $inherit, $return );
		
		// Return
		if ( $return === true ) {
			return $field;
		}
	}
	
	// onCCK_FieldPrepareStore
	public function onCCK_FieldPrepareStore( &$field, $value = '', &$config = array(), $inherit = array(), $return = false )
	{
		if ( self::$type != $field->type ) {
			return;
		}

		// Init
		if ( count( $inherit ) ) {
			$name	=	( isset( $inherit['name'] ) && $inherit['name'] != '' ) ? $inherit['name'] : $field->name;
		} else {
			$name	=	$field->name;
		}
		
		if ( $field->bool == 0 ) {
			$fieldnames	=	explode( '||', $field->options );
			$options2	=	new JRegistry( $field->options2 );
			$latitude	=	$options2->get( 'latitude' );
			$longitude	=	$options2->get( 'longitude' );
			parent::g_addProcess(
				'beforeStore',
				self::$type,
				$config,
				array( 
					// 'api_key'=>trim( $this->params->get( 'api_key_alt', $this->params->get( 'api_key', '' ) ) ),
					'name'=>$name,
					'latitude'=>$latitude,
					'longitude'=>$longitude,
					'fieldnames'=>$fieldnames
				)
			);
		}

		// Set or Return
		if ( $return === true ) {
			return $value;
		}
		$field->value	=	$value;
		parent::g_onCCK_FieldPrepareStore( $field, $name, $value, $config );
	}
	
	// -------- -------- -------- -------- -------- -------- -------- -------- // Render
	
	// onCCK_FieldRenderContent
	public static function onCCK_FieldRenderContent( $field, &$config = array() )
	{
		return parent::g_onCCK_FieldRenderContent( $field );
	}
	
	// onCCK_FieldRenderForm
	public static function onCCK_FieldRenderForm( $field, &$config = array() )
	{
		return parent::g_onCCK_FieldRenderForm( $field );
	}
	
	// -------- -------- -------- -------- -------- -------- -------- -------- // Special Events
	protected static function _getCoordinatesForAddress($address)
    {
        $coordinate = new stdClass();

        $coordinate->latitude = '';
        $coordinate->longitude = '';

       	$photon_url = 'https://photon.komoot.de/api/?limit=1&q='.urlencode($address).'&lang='.substr( JFactory::getLanguage()->getTag(), 0, 2 );

       	$curl = curl_init();

		curl_setopt_array($curl, array(
		  CURLOPT_URL => $photon_url,
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_ENCODING => "",
		  CURLOPT_MAXREDIRS => 10,
		  CURLOPT_TIMEOUT => 300,
		  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		  CURLOPT_CUSTOMREQUEST => 'GET',
		  CURLOPT_HTTPHEADER => array(
		    'Content-Type: application/json',
		  ),
		));

		$response = curl_exec($curl);
		$err = curl_error($curl);

		curl_close($curl);

		if (!$err) {

		  	$photon_data = json_decode($response,true);
		  	// $coordinate->photon_data = $photon_data;

			if(isset($photon_data['features']) && is_array($photon_data['features']) && isset($photon_data['features'][0]) ){

				$feature = $photon_data['features'][0];

				if(isset($feature['geometry']['coordinates'])){

					$coords = $feature['geometry']['coordinates'];

					$coordinate->latitude = $coords[1];
        			$coordinate->longitude = $coords[0];

				}

			}

		}

        return $coordinate;
    }
	// onCCK_FieldBeforeStore
	public static function onCCK_FieldBeforeStore( $process, &$fields, &$storages, &$config = array() )
	{
		$address	=	'';
		if ( count( $process['fieldnames'] ) && $process['latitude'] && $process['longitude'] ) {
			foreach ( $process['fieldnames'] as $name ) {
				$address	.=	$fields[$name]->value.' ';
			}
			$address		=	trim( $address );
			if ( $address ) {

				$coordinate = self::_getCoordinatesForAddress($address);

				$lat		=	$process['latitude'];
				$lng		=	$process['longitude'];
				$fields[$lat]->value	=	$coordinate->latitude;
				$fields[$lng]->value	=	$coordinate->longitude;

				$config['storages'][$fields[$lat]->storage_table][$fields[$lat]->storage_field]	=	$fields[$lat]->value;
				$config['storages'][$fields[$lng]->storage_table][$fields[$lng]->storage_field]	=	$fields[$lng]->value;

			}
		}
	}

	// -------- -------- -------- -------- -------- -------- -------- -------- // Stuff & Script
	
	// onCCK_FieldBeforeRenderForm
	public static function onCCK_FieldBeforeRenderForm( $process, &$fields, &$storages, &$config = array() )
	{
		$name	=	$process['name'];

		if ( !$fields[$name]->state ) {
			return;
		}

		self::_addScripts( $process['id'], $process['variation'], array(
									'api_key'=>trim($process['api_key']),
									'bypass'=>$process['bypass'],
									'lat'=>$process['lat'],
									'lng'=>$process['lng'],
									'postal_code'=>$process['postal_code'],
									'city'=>$process['city'],
									'country'=>$process['country'],
									'country_type'=>$process['country_type'],
									'r_type'=>$process['r_type'],
									'r_country'=>$process['r_country']
								), $config );
	}

	// _addScripts
	protected static function _addScripts( $id, $variation, $params = array(), &$config = array() )
	{
		static $loaded	=	0;

		if ( $params['bypass'] == '0' ) {
			$params['bypass'] == '-1';
		}

		$doc	=	JFactory::getDocument();
		$js2	=	'';
		$opts	=	'types: ['.( $params['r_type'] ? '"'.$params['r_type'].'"' : '' ).'], '
				.	'componentRestrictions: {'.( $params['r_country'] ? 'country: "'.$params['r_country'].'"' : '' ).'}';

		if ( $variation == 'form_filter' || $variation == 'form_filter_ajax' ) {
			if ( isset( $config['submit'] ) && isset( $config['formId'] ) ) {
				$submit		=	$config['submit'];
			} else {
				$submit		=	'JCck.Core.submit';
			}
			$js2			=	$submit.'(\'search\');';
		}

		$plugin_assets_path = '/plugins/cck_field/address_to_coordinates_osm/assets/';

		$doc->addScript(JUri::root() . JPath::clean($plugin_assets_path . '/jquery.autocomplete.js'));
		$doc->addScript(JUri::root() . JPath::clean($plugin_assets_path . '/jquery.autocomplete.min.js'));

		// Add styles
		$osm_style = '

			.autocomplete-suggestions { border: 1px solid #999; background: #FFF; overflow: auto; }
			.autocomplete-suggestion { padding: 5px 5px; cursor: pointer; overflow: hidden; }
			.autocomplete-selected { background: #F0F0F0; }
			.autocomplete-suggestions strong { font-weight: normal; color: #3399FF; }
			.autocomplete-group { padding: 2px 5px; }
			.autocomplete-group strong { display: block; border-bottom: 1px solid #000; }

		';

		$doc->addStyleDeclaration($osm_style);

		$osm_js		=	'
					jQuery(document).ready(function($){

						var $osm_atc_ac = $("#'.$id.'");
						var $osm_atc_el = document.getElementById("'.$id.'");
						var $osm_atc_lat = $("#'.$params['lat'].'");
						var $osm_atc_lat2 = $("#_'.$params['lat'].'");
						var $osm_atc_lng = $("#'.$params['lng'].'");
						var $osm_atc_lng2 = $("#_'.$params['lng'].'");
						var $osm_atc_country = $("#'.@$params['country'].'");
						var $osm_atc_city = $("#'.@$params['city'].'");
						var $osm_atc_zipcode = $("#'.@$params['postal_code'].'");

						jQuery("#'.$id.'").devbridgeAutocomplete({
						    minChars : 3,
						    // width : "flex",
						    lookup: function (query, done) {

								var photon_suggestions = [];

								var photon_url = "https://photon.komoot.de/api/?q="+query+"&lang='.substr( JFactory::getLanguage()->getTag(), 0, 2 ).'";

								console.log(photon_url);

								var currentPhotonRequest = null;

								// Do Ajax call or lookup locally, when done,

								currentPhotonRequest = $.ajax({
									cache: true,
								  	url: photon_url,
								  	beforeSend : function()    {           
								        if(currentPhotonRequest != null) {
								            currentPhotonRequest.abort();
								        }
								    }
								  })
								  .done(function( data ) {

									  	$.each(data.features, function(i, item) {

											var suggestion = {};
											suggestion.feature = item;

											var name        = ("name" in item.properties ? item.properties.name : "" );
											var housenumber = ("housenumber" in item.properties ? item.properties.housenumber : "" );
											var street      = ("street" in item.properties ? item.properties.street : "" );
											var city        = ("city" in item.properties ? item.properties.city : "" );
											var state       = ("state" in item.properties ? item.properties.state : "" );
											var country     = ("country" in item.properties ? item.properties.country : "" );
											var postcode    = ("postcode" in item.properties ? item.properties.postcode : "" );

											var displayValue  = "";
											    displayValue += (name.length > 0 ?  name + ",  " : "" );
												displayValue += (housenumber.length > 0 ? housenumber + ",  " : "" );
												displayValue += (street.length > 0 ? street + ",  " : "" );
												displayValue += (city.length > 0 ? city + ",  " : "" );
												displayValue += (state.length > 0 ? state + ",  " : "" );
												displayValue += (country.length > 0 ? country + ",  " : "" );
												displayValue += (postcode.length > 0 ? postcode + "" : "" );
												displayValue += (displayValue.length > 0 ? "." : "" );

											suggestion.value = displayValue;

											suggestion.name = name;
											suggestion.housenumber = housenumber;
											suggestion.street = street;
											suggestion.city = city;
											suggestion.state = state;
											suggestion.country = country;
											suggestion.postcode = postcode;

											suggestion.osm_id = ("osm_id" in item.properties ? item.properties.osm_id : "" );
											suggestion.osm_type = ("osm_type" in item.properties ? item.properties.osm_type : "" );
											suggestion.osm_key = ("osm_key" in item.properties ? item.properties.osm_key : "" );

											suggestion.coordinates = ("coordinates" in item.geometry ? item.geometry.coordinates : "" );

											photon_suggestions.push(suggestion);

										});
										var result = { suggestions: photon_suggestions };
								        done(result);

						         });
						    },
						    onSelect: function (suggestion) {

								console.log(suggestion);

								if ($osm_atc_zipcode.length) {
									if (suggestion.postcode) {
										$osm_atc_zipcode.val(suggestion.postcode);
									}
								}

								if ($osm_atc_zipcode.length) {
									if (suggestion.postcode) {
										$osm_atc_zipcode.val(suggestion.postcode);
									}
								}

								if ($osm_atc_city.length) {
									if (suggestion.city) {
										$osm_atc_city.val(suggestion.city);
									}
								}

								if ($osm_atc_country.length) {
									if (suggestion.country) {
										$osm_atc_country.val(suggestion.country);
									}
								}

								var latitude = suggestion.coordinates[1];
								var longitude = suggestion.coordinates[0];

								$osm_atc_lat.val(latitude); if ($osm_atc_lat2.length){ $osm_atc_lat2.text(latitude); }
								$osm_atc_lng.val(longitude); if ($osm_atc_lng2.length){ $osm_atc_lng2.text(longitude); }

								'.$js2.'

						    }
						});

						jQuery("#'.$id.'").attr("autocomplete", "osm");

					});
					';

		$doc->addScriptDeclaration( $osm_js );

		if ( $loaded ) {
			return;
		}

		$loaded		=	1;		

	}
}
?>
